/*
 * gsampler-client-device.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdlib.h>
#include <string.h>

#include "gsampler-client-device.h"
#include "gsampler-utils.h"

void 
gsampler_device_info_copy (const GsamplerDeviceInfo* self, GsamplerDeviceInfo* dest) 
{
	dest->id = self->id;
	dest->active = self->active;
	dest->type = self->type;
	dest->driver = g_strdup (self->driver);
	dest->name = g_strdup (self->name);
	dest->param = self->param;
	dest->samplerate = self->samplerate;
	dest->fragments = self->fragments;
	dest->fragmentsize = self->fragmentsize;
}

GsamplerDeviceInfo* 
gsampler_device_info_dup (const GsamplerDeviceInfo* self) 
{
	GsamplerDeviceInfo* dup;
	dup = g_new0 (GsamplerDeviceInfo, 1);
	gsampler_device_info_copy (self, dup);
	return dup;
}

void 
gsampler_device_info_destroy (GsamplerDeviceInfo* self) 
{
	g_free (self->driver);
	g_free (self->name);
}

void 
gsampler_device_info_free (GsamplerDeviceInfo* self) 
{
	gsampler_device_info_destroy (self);
	g_free (self);
}

/**
 * 
 * Device Binding structure
 * 
 **/

GsamplerDeviceBinding*
gsampler_device_binding_dup (const GsamplerDeviceBinding* self) 
{
	GsamplerDeviceBinding* dup;
	dup = g_new0 (GsamplerDeviceBinding, 1);
	dup->type = g_strdup (self->type);
	dup->name = g_strdup (self->name);
	dup->active = self->active;
	return dup;
}

void
gsampler_device_binding_destroy (GsamplerDeviceBinding* self) 
{
	g_free (self->type);
	g_free (self->name);
	
}

void 
gsampler_device_binding_free (GsamplerDeviceBinding* self) 
{	
	gsampler_device_binding_destroy (self);
	g_free (self);
}

/**
 *  Device Routine
 * 
 **/

static gint 
gsampler_client_create_device (GsamplerClient* self, const gchar* com,
                               const gchar* type, const gchar* name) {
	gint result = -1;
	gchar* send;
	g_return_val_if_fail (self != NULL, -1);							   
	g_return_val_if_fail (com != NULL, -1);
	g_return_val_if_fail (type != NULL, -1);
	g_return_val_if_fail (name != NULL, -1);
	
	if (strlen (name) != 0) {
		send = g_strdup_printf ("%s %s NAME='%s'", com, type, name);
	} else {
		send = g_strdup_printf ("%s %s", com, type);
	}
	result = gsampler_client_create (self, send);
	g_free (send);
	return result;
}

gint 
gsampler_client_create_audio_device (GsamplerClient* self, const gchar* type, const gchar* name) {
	return gsampler_client_create_device (self, "CREATE AUDIO_OUTPUT_DEVICE", type, name);
}

gint 
gsampler_client_create_midi_device (GsamplerClient* self, const gchar* type, const gchar* name) {
	return gsampler_client_create_device (self, "CREATE MIDI_INPUT_DEVICE", type, name);
}

gboolean 
gsampler_client_delete_audio_device (GsamplerClient* self, gint id) {
	return gsampler_client_send_bool (self, g_strdup_printf ("DESTROY AUDIO_OUTPUT_DEVICE %i", id));
}

gboolean 
gsampler_client_delete_midi_device (GsamplerClient* self, gint id) {
	return gsampler_client_send_bool (self, g_strdup_printf ("DESTROY MIDI_INPUT_DEVICE %i", id));
}

gint 
gsampler_client_get_audio_output_count (GsamplerClient* self) {
	return gsampler_client_get_int (self, "GET AUDIO_OUTPUT_DEVICES");
}

gint 
gsampler_client_get_midi_input_count (GsamplerClient* self) {
	return gsampler_client_get_int (self, "GET MIDI_INPUT_DEVICES");
}

gchar** 
gsampler_client_get_available_audio_output (GsamplerClient* self) {
	return gsampler_client_get_list (self, "LIST AUDIO_OUTPUT_DEVICES");
}

gchar** 
gsampler_client_get_available_midi_input (GsamplerClient* self) {
	return gsampler_client_get_list (self, "LIST MIDI_INPUT_DEVICES");
}

gchar** 
gsampler_client_get_audio_driver_list (GsamplerClient* self) {
	return gsampler_client_get_list (self, "LIST AVAILABLE_AUDIO_OUTPUT_DRIVERS");
}

gchar** 
gsampler_client_get_midi_driver_list (GsamplerClient* self) {
	return gsampler_client_get_list (self, "LIST AVAILABLE_MIDI_INPUT_DRIVERS");
}

GsamplerDeviceInfo * 
gsampler_client_get_device_info (GsamplerClient* self, gint id, gint type) {
	gchar* com = NULL;
	GsamplerDeviceInfo *result = NULL;
	g_return_val_if_fail (self != NULL, NULL);
									 
	result = g_new0 (GsamplerDeviceInfo, 1);
	if (type == 0) {
		com = "GET AUDIO_OUTPUT_DEVICE INFO";
	} else if (type == 1) {
		com = "GET MIDI_INPUT_DEVICE INFO";
	} else {
		g_warning ("Invalide device type : %d", type);
		result->type = -1;
		return result;
	}
	if (com != NULL) {
		GSList* ret = NULL;
		com = g_strdup_printf ("%s %i",com, id);
		ret = gsampler_client_client_get_info (self, com);
		if (g_slist_length (ret) != 0) {
			result->id = id;
			result->type = type;
			GSList* element_it;
			for (element_it = ret; element_it != NULL; element_it = element_it->next) {
				gchar* element = NULL;
				gchar** tmp = NULL;
				element = g_strdup ((const gchar*) element_it->data);
				if (string_contains (element, "ACTIVE")) {
					tmp = g_strsplit (element, ":", 0);
					result->active = bool_parse (g_strstrip (tmp[1]));
				} else if (string_contains (element, "DRIVER")) {
					tmp = g_strsplit (element, ":", 0);
					result->driver = g_strstrip (g_strdup(tmp[1]));
				} else if (string_contains (element, "NAME")) {
					tmp = g_strsplit (element, ":", 0);
					result->name = g_strstrip (string_replace (tmp[1], "'", ""));
				} else if (string_contains (element, "CHANNELS")) {
					tmp = g_strsplit (element, ":", 0);
					result->param = atoi (g_strstrip (tmp[1]));
				} else if (string_contains (element, "PORTS")) {
					tmp = g_strsplit (element, ":", 0);
					result->param = atoi (g_strstrip (tmp[1]));
				} else if (string_contains (element, "SAMPLERATE")) {
					tmp = g_strsplit (element, ":", 0);
					result->samplerate = double_parse (g_strstrip (tmp[1]));
				} else if (string_contains (element, "FRAGMENTS")) {
					tmp = g_strsplit (element, ":", 0);
					result->fragments = atoi (g_strstrip (tmp[1]));
				} else if (string_contains (element, "FRAGMENTSIZE")) {
					tmp = g_strsplit (element, ":", 0);
					result->fragmentsize = double_parse (g_strstrip (tmp[1]));
				}
				g_free (element);
				g_strfreev (tmp);
			}
		} else {
			result->type = -1;
		}
		g_slist_free_full (ret, g_free);
		g_free (com);
	} else {
		result->type = -1;
	}
	return result;
}

gboolean 
gsampler_client_set_audio_output_parameter (GsamplerClient* self, gint id, const gchar* type,
                                            const gchar* val) {
	g_return_val_if_fail (val != NULL, FALSE);

	return gsampler_client_send_bool (self, g_strdup_printf ("SET AUDIO_OUTPUT_DEVICE_PARAMETER %i %s=%s",
	                                                         id, type, val));
}

gboolean 
gsampler_client_set_midi_input_parameter (GsamplerClient* self, gint id, const gchar* type,
                                          const gchar* val) {
	g_return_val_if_fail (val != NULL, FALSE);

	return gsampler_client_send_bool (self, g_strdup_printf ("SET MIDI_INPUT_DEVICE_PARAMETER %i %s=%s",
	                                                         id, type, val));
}

GHashTable* 
gsampler_client_get_audio_output_channel_info (GsamplerClient* self, gint id, gint chan) {
	GHashTable* result;
	gchar* com = NULL;
	g_return_val_if_fail (self != NULL, NULL);

	com = g_strdup_printf ("GET AUDIO_OUTPUT_CHANNEL INFO %i %i", id, chan);
	result = g_hash_table_new_full (g_str_hash,  g_str_equal, g_free, g_free);
	if (com != NULL) {
		GSList *info = NULL;
		GSList *element_it;
		info = gsampler_client_client_get_info (self, com);
		if (g_slist_length (info) != 0) {
			for (element_it = info; element_it != NULL; element_it = element_it->next) {
				gchar* element = NULL;
				gchar* element_key = NULL;
				gchar* element_val = NULL;
				gchar** info;
				element = g_strdup ((gchar*) element_it->data);
				info = g_strsplit (element, ":", 2);
				element_key = g_strdup (info[0]);
				element_val = string_replace (info[1], "'", "");
				g_hash_table_insert (result, g_strdup (element_key),
				                     g_strdup (g_strstrip (element_val)));
				g_strfreev (info);
				g_free (element);
				g_free (element_key);
				g_free (element_val);
			}
			g_slist_free_full (info, g_free);
		}
	}
	g_free (com);
	return result;
}

gchar** 
gsampler_client_get_audio_output_channel_parameter_info (GsamplerClient* self, gint id, gint chan, 
                                                         const gchar* param) {
	gchar** result = NULL;
	gchar* com = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (param != NULL, NULL);

	com = g_strdup_printf ("GET AUDIO_OUTPUT_CHANNEL_PARAMETER INFO %i %i %s", id, chan, param);
	if (com != NULL) {
		GSList* ret;
		GSList* element_it;
		ret = gsampler_client_client_get_info (self, com);
		for (element_it = ret; element_it != NULL; element_it = element_it->next) {
			gchar* element;
			element = g_strdup ((const gchar*) element_it->data);
			if (string_contains (element, "POSSIBILITIES")) {				
				element = string_replace (element, "POSSIBILITIES:", "");
				element = string_replace (element, "'", "");
				result = g_strsplit (g_strstrip (element), ",", 0);
			}
			g_free (element);
		}
		g_slist_free_full (ret, g_free);
		
	}
	g_free (com);
	return result;
}

gboolean 
gsampler_client_set_audio_output_channel_parameter (GsamplerClient* self, gint id, gint chan, 
                                                    const gchar* param, const gchar* val) {
	g_return_val_if_fail (param != NULL, FALSE);
	g_return_val_if_fail (val != NULL, FALSE);

	return gsampler_client_send_bool (self, g_strdup_printf ("SET AUDIO_OUTPUT_CHANNEL_PARAMETER %i %i %s='%s'",
	                                                         id, chan, param, val));
}

GHashTable* 
gsampler_client_get_midi_input_port_info (GsamplerClient* self, gint id, gint port) {
	GHashTable* result;
	gchar* com = NULL;
	g_return_val_if_fail (self != NULL, NULL);

	com = g_strdup_printf ("GET MIDI_INPUT_PORT INFO %i %i", id, port);
	result = g_hash_table_new_full (g_str_hash,  g_str_equal, g_free, g_free);
	if (com != NULL) {
		GSList *info = NULL;
		GSList *element_it;
		info = gsampler_client_client_get_info (self, com);
		if (g_slist_length (info) != 0) {
			for (element_it = info; element_it != NULL; element_it = element_it->next) {
				gchar* element = NULL;
				gchar* element_key = NULL;
				gchar* element_val = NULL;
				gchar** info;
				element = g_strdup ((const gchar*) element_it->data);
				info = g_strsplit (element, ":", 2);
				element_key = g_strdup (info[0]);
				element_val = string_replace (info[1], "'", "");
				g_hash_table_insert (result, g_strdup (element_key), 
				                     g_strdup (g_strstrip (element_val)));
				g_strfreev (info);
				g_free (element);
				g_free (element_key);
				g_free (element_val);
			}
			g_slist_free_full (info, g_free);
		}
	}
	g_free (com);
	return result;
}

gchar** 
gsampler_client_get_midi_input_port_parameter_info (GsamplerClient* self, gint id, gint port, 
                                                         const gchar* param) {
	gchar** result = NULL;
	gchar* com = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (param != NULL, NULL);

	com = g_strconcat ("GET MIDI_INPUT_PORT_PARAMETER INFO ", g_strdup_printf ("%i", id),
	                      " ", g_strdup_printf ("%i", port), " ", param, NULL);

	if (com != NULL) {
		GSList* ret = NULL;
		GSList* element_it;
		ret = gsampler_client_client_get_info (self, com);
		for (element_it = ret; element_it != NULL; element_it = element_it->next) {
			gchar* element;
			element = g_strdup ((const gchar*) element_it->data);
			if (string_contains (element, "POSSIBILITIES")) {				
				element = string_replace (element, "POSSIBILITIES:", "");
				element = string_replace (element, "'", "");
				element = g_strstrip (element);
				result = g_strsplit (element, ",", 0);
			}
			g_free (element);
		}
		g_slist_free_full (ret, g_free);
		g_free (com);
	}
	return result;
}

gboolean 
gsampler_client_set_midi_input_port_parameter (GsamplerClient* self, gint id, gint chan, 
                                                    const gchar* param, const gchar* val) {
	g_return_val_if_fail (param != NULL, FALSE);
	g_return_val_if_fail (val != NULL, FALSE);

	return gsampler_client_send_bool (self, g_strdup_printf ("SET MIDI_INPUT_PORT_PARAMETER %i %i %s='%s'",
	                                                         id, chan, param, val));
}

GSList* 
gsampler_client_get_avail_device_parameter_bindings (GsamplerClient* self, gint type, gint id, gint chan_id) {
	GHashTable* list;
	GHashTableIter iter;
	gpointer key, value;
	GSList* ret = NULL;
	g_return_val_if_fail (self != NULL, NULL);

	if (type == 0) {
		list = gsampler_client_get_audio_output_channel_info (self, id, chan_id);
	} else if ( type == 1) {
		list = gsampler_client_get_midi_input_port_info (self, id, chan_id);
	} else {
		return NULL;
	}
	if (g_hash_table_size (list) != 0) {
		g_hash_table_iter_init (&iter, list);
		while (g_hash_table_iter_next (&iter, &key, &value)) {
			gchar* entrie_key = NULL;
			gchar* entrie_val = NULL;

			entrie_val = g_strdup ((gchar*) value);
			entrie_key = g_strdup ((gchar*) key);
			if (string_contains (entrie_key, "BINDINGS")) {
				gchar** vals = NULL;
				gint vals_lenght = 0;
				gint val_it;
				if (type == 0) {
					vals = gsampler_client_get_audio_output_channel_parameter_info (self, id, chan_id, entrie_key);
				} else if (type == 1) {
					vals = gsampler_client_get_midi_input_port_parameter_info (self, id, chan_id, entrie_key);
				}
				if (vals != NULL)
					vals_lenght = g_strv_length (vals);
				for (val_it = 0; val_it < vals_lenght; val_it = val_it + 1) {
					gboolean status = FALSE;
					GsamplerDeviceBinding* binding = g_new (GsamplerDeviceBinding, 1);
					binding->type = g_strdup (entrie_key);
					binding->name = g_strdup (vals[val_it]);
					if (g_strcmp0 (binding->name, entrie_val) == 0) {
						status = TRUE;
					}
					binding->active = status;
					ret = g_slist_prepend (ret, binding);	
				}
				ret = g_slist_reverse (ret);
				g_strfreev (vals);
			}
			g_free (entrie_val);
			g_free (entrie_key);
		}
	}
	g_hash_table_destroy (list);
	return ret;
 }
