/*
 * gsampler-client-midimap.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <string.h>

#include "gsampler-client-midimap.h"
#include "gsampler-utils.h"

/**
 * 
 * Midi Map Info structure
 * 
 **/
void 
gsampler_midi_map_info_copy (const GsamplerMidiMapInfo* self, GsamplerMidiMapInfo* dest) 
{
	dest->name = g_strdup (self->name);
	dest->defaut = self->defaut;
}

GsamplerMidiMapInfo* 
gsampler_midi_map_info_dup (const GsamplerMidiMapInfo* self) 
{
	GsamplerMidiMapInfo* dup;
	dup = g_new0 (GsamplerMidiMapInfo, 1);
	gsampler_midi_map_info_copy (self, dup);
	return dup;
}

void 
gsampler_midi_map_info_destroy (GsamplerMidiMapInfo* self) 
{
	g_free (self->name);
}

void 
gsampler_midi_map_info_free (GsamplerMidiMapInfo* self) 
{
	gsampler_midi_map_info_destroy (self);
	g_free (self);
}

// MIDI INST INFO
void gsampler_midi_inst_info_copy (const GsamplerMidiInstInfo* self, GsamplerMidiInstInfo* dest) 
{
	dest->name = g_strdup (self->name);
	dest->engine = g_strdup (self->engine);
	dest->file = g_strdup (self->file);
	dest->inst_nb = self->inst_nb;
	dest->inst_name = g_strdup (self->inst_name);
	dest->load_mode = g_strdup (self->load_mode);
	dest->vol = self->vol;
}

GsamplerMidiInstInfo* 
gsampler_midi_inst_info_dup (const GsamplerMidiInstInfo* self) 
{
	GsamplerMidiInstInfo* dup;
	dup = g_new0 (GsamplerMidiInstInfo, 1);
	gsampler_midi_inst_info_copy (self, dup);
	return dup;
}

void 
gsampler_midi_inst_info_destroy (GsamplerMidiInstInfo* self) 
{
	g_free ((*self).name);
	g_free ((*self).engine);
	g_free ((*self).file);
	g_free ((*self).inst_name);
	g_free ((*self).load_mode);
}

void
gsampler_midi_inst_info_free (GsamplerMidiInstInfo* self)
{
	gsampler_midi_inst_info_destroy (self);
	g_free (self);
}

/**
 * MIDI MAP STUFF
 * 
 **/
gint 
gsampler_client_create_midi_instrument_map (GsamplerClient* self, const gchar* name) 
{
	g_return_val_if_fail (self != NULL, -1);

	return gsampler_client_create (self, g_strdup_printf ("ADD MIDI_INSTRUMENT_MAP '%s'", name));
}

gboolean 
gsampler_client_delete_midi_instrument_map (GsamplerClient* self, gint id) 
{
	g_return_val_if_fail (self != NULL, FALSE);
	
	return gsampler_client_send_bool (self, g_strdup_printf ("REMOVE MIDI_INSTRUMENT_MAP %i", id));
}

gboolean 
gsampler_client_delete_all_midi_instrument_map (GsamplerClient* self) 
{
	g_return_val_if_fail (self != NULL, FALSE);
	
	return gsampler_client_send_bool (self, "REMOVE MIDI_INSTRUMENT_MAP ALL");
}

gint 
gsampler_client_get_midi_instrument_map (GsamplerClient* self) {
	return gsampler_client_get_int (self, "GET MIDI_INSTRUMENT_MAPS");
}

gchar** 
gsampler_client_get_available_midi_instrument_map (GsamplerClient* self) 
{
	return gsampler_client_get_list (self, "LIST MIDI_INSTRUMENT_MAPS");
}

GsamplerMidiMapInfo * 
gsampler_client_get_midi_instrument_map_info (GsamplerClient* self, gint id) 
{
	GsamplerMidiMapInfo *retour = NULL;
	GSList* ret;
	g_return_val_if_fail (self != NULL, NULL);

	retour = g_new0 (GsamplerMidiMapInfo, 1);
	ret = gsampler_client_client_get_info (self, g_strdup_printf ("GET MIDI_INSTRUMENT_MAP INFO %i", id));
	if (g_slist_length (ret) != 0) 
	{
		GSList* ele_it;
		for (ele_it = ret; ele_it != NULL; ele_it = ele_it->next) 
		{
			gchar* ele;
			ele = g_strdup ((const gchar*) ele_it->data);
			if (string_contains (ele, "NAME")) 
			{
				retour->name = string_replace (ele, "NAME: ", "");
			} else if (string_contains (ele, "DEFAULT"))
			{
				gchar* temp;
				temp = string_replace (ele, "DEFAULT: ", "");
				retour->defaut = bool_parse (temp);
				g_free (temp);		
			}
			g_free (ele);
		}
	} else 
	{
		retour->name = g_strdup ("NONE");
	}
	g_slist_free_full (ret, g_free);
	return retour;
}

gboolean 
gsampler_client_set_midi_instrument_map_name (GsamplerClient* self, gint id, const gchar* name) 
{
	g_return_val_if_fail (self != NULL, FALSE);
	
	return gsampler_client_send_bool (self, g_strdup_printf ("SET MIDI_INSTRUMENT_MAP NAME %i '%s'", id, name));
}

gboolean 
gsampler_client_map_midi_instrument (GsamplerClient* self, gint map, gdouble bank, gint prog,
                                     const gchar* engine, const gchar* file, gint instrument, 
                                     gdouble vol, const gchar* load_mode,  const gchar* name) 
{
	gchar* temp;
	gboolean result = FALSE;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (engine != NULL, FALSE);
	g_return_val_if_fail (file != NULL, FALSE);

	temp = g_strdup_printf ("MAP MIDI_INSTRUMENT NON_MODAL %i %i %i %s '%s' %i %s",
	                        map, (int) bank, prog, engine, file, instrument, double_to_string ((vol/100)));	
	if (load_mode != NULL && g_strcmp0 (load_mode, "Default") != 0) 
	{
		temp = g_strconcat (temp, " ", load_mode, NULL);
	}
	if (name != NULL) 
	{
		temp = g_strconcat (temp, " '", name, "'", NULL);
	}
	if (temp != NULL)
		result = gsampler_client_send_bool (self, temp);
	g_free (temp);
	return result;
}

gchar* 
gsampler_client_get_midi_instruments (GsamplerClient* self, gint map) 
{
	g_return_val_if_fail (self != NULL, 0);
	
	return gsampler_client_get_char (self, g_strdup_printf ("GET MIDI_INSTRUMENTS %i", map));
}

GSList *
gsampler_client_get_available_midi_instruments (GsamplerClient* self, gint map) 
{
	GSList *instlist = NULL;
	gchar* ret;
	g_return_val_if_fail (self != NULL, NULL);

	ret = gsampler_client_send_commande (self, g_strdup_printf ("LIST MIDI_INSTRUMENTS %i", map));
	if (strlen (ret) != 0) 
	{
		GRegex *regex;
		GMatchInfo *match_info;
		regex = g_regex_new ("{(.*?)}", 0, 0, NULL);
		g_regex_match (regex, ret, 0, &match_info);
		while (g_match_info_matches (match_info)) 
		{
			instlist = g_slist_prepend (instlist, g_match_info_fetch (match_info, 1));
			g_match_info_next (match_info, NULL);
		}
		instlist = g_slist_reverse (instlist);

		g_match_info_free (match_info);
		g_regex_unref (regex);
	}
	g_free (ret);
	return instlist;
}

gboolean 
gsampler_client_unmap_midi_instrument (GsamplerClient* self, gint map, gint bank, gint prog) 
{
	g_return_val_if_fail (self != NULL, FALSE);

	return gsampler_client_send_bool (self, g_strdup_printf ("UNMAP MIDI_INSTRUMENT %i %i %i",
	                                                         map, bank, prog));
}

GsamplerMidiInstInfo* 
gsampler_client_get_midi_instruments_info (GsamplerClient* self, gint map, gint bank, gint prog) 
{
	GsamplerMidiInstInfo* result = NULL;
	gchar* com;
	GSList* ret;
	g_return_val_if_fail (self != NULL, NULL);

	result = g_new0 (GsamplerMidiInstInfo, 1);
	com = g_strdup_printf ("GET MIDI_INSTRUMENT INFO %i %i %i", map, bank, prog);
	ret = gsampler_client_client_get_info (self, com);
	if (g_slist_length (ret) != 0) 
	{
		GSList* ele_it;
		for (ele_it = ret; ele_it != NULL; ele_it = ele_it->next)
		{
			gchar* ele;
			ele = g_strdup ((const gchar*) ele_it->data);
			if (string_contains (ele, "ENGINE_NAME"))
			{
				result->engine = string_replace (ele, "ENGINE_NAME: ", "");
			} else if (string_contains (ele, "INSTRUMENT_FILE"))
			{
				gchar* test;
				test = string_replace (ele, "INSTRUMENT_FILE: ", "");
				result->file = string_replace (test, "\\x20", " ");
				g_free (test);
			} else if (string_contains (ele, "INSTRUMENT_NR"))
			{
				gchar* temp;
				temp = string_replace (ele, "INSTRUMENT_NR: ", "");
				result->inst_nb = atoi (temp);
				g_free (temp);
			} else if (string_contains (ele, "INSTRUMENT_NAME"))
			{
				result->inst_name = g_strstrip (string_replace (ele, "INSTRUMENT_NAME:", ""));
			} else if (string_contains (ele, "LOAD_MODE"))
			{
				result->load_mode = string_replace (ele, "LOAD_MODE: ", "");
			} else if (string_contains (ele, "VOLUME"))
			{
				gchar* temp;
				temp = string_replace (ele, "VOLUME: ", "");
				result->vol = double_parse (temp);
				g_free (temp);
			} else if (string_contains (ele, "NAME"))
			{
				result->name = string_replace (ele, "NAME: ", "");
			}
			g_free (ele);
		}
	} else
	{
		result->name = g_strdup ("NONE");
	}
	g_slist_free_full (ret, g_free);
	g_free (com);
	return result;
}

gboolean 
gsampler_client_clear_midi_instrument_map (GsamplerClient* self, gint id) 
{
	gboolean result = FALSE;
	gchar* tmp = NULL;
	g_return_val_if_fail (self != NULL, FALSE);
	if (id == (-1))
	{
		tmp = g_strdup ("ALL");
	} else
	{
		tmp = g_strdup_printf ("%i", id);
	}
	result = gsampler_client_send_bool (self , g_strconcat ("CLEAR MIDI_INSTRUMENTS ", tmp, NULL));
	g_free (tmp);
	return result;
}