/*
 * gsampler-database-search.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 *
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib/gi18n-lib.h>

#include "gsampler-window.h"
#include "gsampler-database-search.h"
#include "gsampler-client-database.h"
#include "gsampler-utils.h"

enum {
	SEARCH_SELECT_EVENT,
	SEARCH_NB_SIGNALS
};

static guint search_signals[SEARCH_NB_SIGNALS] = { 0 };

G_DEFINE_TYPE (GsamplerDatabaseSearch, gsampler_database_search, GTK_TYPE_DIALOG);

static void
gsampler_database_search_select_event (GtkTreeSelection *select, gpointer data)
{
	GtkTreeIter iter = {0};
	GtkTreeModel* model = NULL;
	GsamplerDatabaseSearch* self = (GsamplerDatabaseSearch *) data;
	g_return_if_fail (self != NULL);
	g_return_if_fail (select != NULL);
	
	if (gtk_tree_selection_get_selected (select, &model, &iter)) 
	{
		gchar* name;
		gtk_tree_model_get (model, &iter, 0, &name, -1);
		g_signal_emit (self, search_signals[SEARCH_SELECT_EVENT], 0, name);
	}
}

static gboolean
gsampler_database_search_find_event (GtkButton *button, gpointer data)
{
	GsamplerClient* client = NULL;
	gchar* dir = NULL;
	gchar* name = NULL;
	GtkTreeIter iter = {0};
	GtkListStore *list = NULL;
	GsamplerDatabaseSearch *self = (GsamplerDatabaseSearch *) data;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (button != NULL, FALSE);
	

	client = gsampler_client_get_instance ();
	list = (GtkListStore*) gtk_tree_view_get_model (self->result_view);
	gtk_list_store_clear (list);
	dir = g_strdup((gchar*) gtk_entry_get_text ((GtkEntry*) self->dir_entry));
	name = g_strdup((gchar*) gtk_entry_get_text ((GtkEntry*) self->find_entry));
	if (dir != NULL && name != NULL && strlen (dir) != 0 && strlen (name) != 0)
	{
		gchar** result = NULL;
		result = gsampler_client_db_find_instrument (client, dir, name);
		if (result != NULL) 
		{
			gint result_length = g_strv_length (result);
			gint rep_it;
			for (rep_it = 0; rep_it < result_length; rep_it = rep_it + 1) 
			{
				gchar* rep = string_replace (result[rep_it], "'", "");
				gtk_list_store_append (list, &iter);
				gtk_list_store_set (list, &iter, 0, rep, -1);
				g_free (rep);
			}
		} else
		{
			gtk_list_store_append (list, &iter);
			gtk_list_store_set (list, &iter, 0, _("No result"), -1);
		}
		g_strfreev (result);
	} else
	{
			gtk_list_store_append (list, &iter);
			gtk_list_store_set (list, &iter, 0, _("Please enter a search dir and name"), -1);
	}
	g_free (dir);
	g_free (name);
	g_object_unref (client);
	return FALSE;
}

static GObject *
gsampler_database_search_constructor (GType type, guint n_construct_properties, 
                              GObjectConstructParam * construct_properties) 
{
	GObject *obj;
	GtkBox* vbox;
	GtkBox* hbox1;
	GtkBox* hbox2;
	GtkWidget* dir_label;
	GtkWidget* find_label;
	GtkWidget* find_button;
	GtkWidget* scroll;
	GsamplerDatabaseSearch *self;
	GsamplerMainWindow* win;
	
	obj = G_OBJECT_CLASS (gsampler_database_search_parent_class)->constructor (type, n_construct_properties, construct_properties);
	self = GSAMPLER_DATABASE_SEARCH (obj);

	win = gsampler_main_window_get_instance ();
	gtk_window_set_transient_for ((GtkWindow*) self, (GtkWindow*) win);
	gtk_window_set_title ((GtkWindow*) self, _("Search an instrument"));
	gtk_container_set_border_width ((GtkContainer*) self, 5);
	gtk_dialog_add_button ((GtkDialog*) self, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
	
	vbox = (GtkBox*) gtk_dialog_get_content_area ((GtkDialog*) self);
	hbox1 = (GtkBox*) gtk_hbox_new (TRUE, 5);
	hbox2 = (GtkBox*) gtk_hbox_new (TRUE, 5);
	self->find_entry = gtk_entry_new ();
	find_label = gtk_label_new (_("Search name : "));
	self->dir_entry = gtk_entry_new ();
	gtk_entry_set_text ((GtkEntry*) self->dir_entry, "/");
	dir_label = gtk_label_new (_("Select base dir : "));
	find_button = gtk_button_new_from_stock (GTK_STOCK_FIND);
	GtkListStore *liste = gtk_list_store_new (1, G_TYPE_STRING);
	self->result_view = (GtkTreeView*) gtk_tree_view_new_with_model ((GtkTreeModel*) liste);
	gtk_tree_view_set_headers_visible (self->result_view, FALSE);
	gtk_tree_view_insert_column_with_attributes (self->result_view, -1, NULL,
	                                             (GtkCellRenderer*) gtk_cell_renderer_text_new (),
	                                             "text", 0, NULL, NULL);	
	scroll = gtk_scrolled_window_new (NULL, NULL);
	//gtk_scrolled_window_set_min_content_height ((GtkScrolledWindow*) scroll, 300);
	//gtk_scrolled_window_set_min_content_width ((GtkScrolledWindow*) scroll, 400);
	gtk_container_add ((GtkContainer*) scroll, (GtkWidget*) self->result_view);
	gtk_box_pack_start (hbox1, dir_label, FALSE, TRUE, 0);
	gtk_box_pack_start (hbox1, self->dir_entry, FALSE, TRUE, 0);
	gtk_box_pack_start (hbox2, find_label, FALSE, TRUE, 0);
	gtk_box_pack_start (hbox2, self->find_entry, FALSE, TRUE, 0);
	gtk_box_pack_start (vbox, (GtkWidget*) hbox1, FALSE, TRUE, 5);
	gtk_box_pack_start (vbox, (GtkWidget*) hbox2, FALSE, TRUE, 5);
	gtk_box_pack_start (vbox, find_button, FALSE, TRUE, 5);
	gtk_box_pack_start (vbox, scroll, TRUE, TRUE, 5);

	GtkTreeSelection *select = gtk_tree_view_get_selection (self->result_view);
	g_signal_connect_object (select, "changed",
	                         (GCallback) gsampler_database_search_select_event, self, 0);
	g_signal_connect_object ((GtkButton*) find_button, "clicked", 
	                         (GCallback) gsampler_database_search_find_event, self, 0);
	g_object_unref (win);
	return obj;
}

static void
gsampler_database_search_init (GsamplerDatabaseSearch *self)
{
	self->result = NULL;
}

static void
gsampler_database_search_finalize (GObject *object)
{
	/* TODO: Add deinitalization code here */

	G_OBJECT_CLASS (gsampler_database_search_parent_class)->finalize (object);
}

static void
gsampler_database_search_class_init (GsamplerDatabaseSearchClass *klass)
{
	gsampler_database_search_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->constructor = gsampler_database_search_constructor;
	G_OBJECT_CLASS (klass)->finalize = gsampler_database_search_finalize;

	search_signals[SEARCH_SELECT_EVENT] = g_signal_new ("gsampler-database-search-select",
	                                                    GSAMPLER_TYPE_DATABASE_SEARCH,
	                                                    G_SIGNAL_RUN_LAST,
	                                                    0, NULL, NULL, 
	                                                    g_cclosure_marshal_VOID__STRING,
	                                                    G_TYPE_NONE,
	                                                    1, G_TYPE_STRING);
}

GsamplerDatabaseSearch* 
gsampler_database_search_new (void) 
{
	return g_object_new (GSAMPLER_TYPE_DATABASE_SEARCH, NULL);
}
