/*
 * gsampler-utils.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <string.h>

#include "gsampler-utils.h"

/*
 * bool and double utility 
 */
gboolean 
bool_parse (const gchar* str) {
	g_return_val_if_fail (str != NULL, FALSE);
	
	if (g_strcmp0 (str, "true") == 0) {
		return TRUE;
	} else {
		return FALSE;
	}
}


gdouble 
double_parse (const gchar* str) {
	gdouble result = 0.0;
	g_return_val_if_fail (str != NULL, 0.0);
	
	result = g_ascii_strtod (str, NULL);
	return result;
}

gchar* 
double_to_string (gdouble value) {
	gchar* temp = NULL;

	temp = g_ascii_dtostr (g_new0 (gchar, G_ASCII_DTOSTR_BUF_SIZE),
	                         G_ASCII_DTOSTR_BUF_SIZE, value);
	return temp;
}

gchar* 
bool_to_string (gboolean self) {
	gchar* result = NULL;
	if (self) {
		result = g_strdup ("true");
	} else {
		result = g_strdup ("false");
	}
	return result;
}

/*
 *  String utility
 */

gboolean 
string_contains (const gchar* str1, const gchar* str2) {
	g_return_val_if_fail (str1 != NULL, FALSE);
	
	return g_strstr_len ((gchar*) str1, strlen (str1), (gchar*) str2) != NULL;
}

/**
   * Replace in string the old with the replacement
   *
   * @param string, old value, new value
   *
   * @return new string
   */
gchar* 
string_replace (const gchar* string, const gchar* old, const gchar* replacement) {
	gchar* result = NULL;
	GRegex* regex;
	GError * error = NULL;
	g_return_val_if_fail (string != NULL, NULL);
	g_return_val_if_fail (old != NULL, NULL);
	g_return_val_if_fail (replacement != NULL, NULL);

	regex = g_regex_new (g_regex_escape_string (old, -1), 0, 0, &error);
	if (error != NULL) {
		g_warning ("file %s: line %d: unexpected error: %s", __FILE__, __LINE__, error->message);
		g_assert_not_reached ();
		g_error_free (error);
		return NULL;
	}
	result = g_regex_replace_literal (regex, string, (gssize) (-1), 0, replacement, 0, &error);
	if (error != NULL) {
		g_regex_unref (regex);
		g_warning ("file %s: line %d: unexpected error: %s", __FILE__, __LINE__, error->message);
		g_assert_not_reached ();
		g_error_free (error);
		return NULL;
	}
	g_regex_unref (regex);
	return result;
}

/**
   * Extracts a slice, or substring, of the specified string
   *
   * @param string, start, end
   *
   * @return new string
   */
gchar* 
string_slice (const gchar* self, glong start, glong end) {
	gchar* result = NULL;
	glong string_length;
	g_return_val_if_fail (self != NULL, NULL);
	
	string_length = (glong) strlen (self);
	if (start < 0) {
		start = string_length + start;
	}
	if (end < 0) {
		end = string_length + end;
	}
	if (start >= 0 && start <= string_length) {
		if (end >= 0 && end <= string_length) {
			if (start <= end) {
				result = g_strndup (((gchar*) self) + start, (gsize) (end - start));
			}
		}
	}	
	return result;
}

static gint 
string_last_index_of_char (const gchar* self, gunichar c, gint start_index) {
	gint result = 0;
	gchar* ret;
	g_return_val_if_fail (self != NULL, 0);

	ret = g_utf8_strrchr (((gchar*) self) + start_index, (gssize) (-1), c);
	if (ret != NULL) {
		result = (gint) (ret - ((gchar*) self));
	} else {
		result = -1;
	}
	return result;
}


static gchar* 
string_substring (const gchar* self, glong offset, glong len) {
	gchar* result = NULL;
	glong string_length = 0L;
	g_return_val_if_fail (self != NULL, NULL);
	
	if (offset >= 0 && len >= 0) {
		string_length = g_utf8_strlen ((gchar*) self, (gssize) offset + len);
	} else {
		string_length = (glong) strlen (self);
	}
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	result = g_strndup (((gchar*) self) + offset, (gsize) len);
	return result;
}

/**
   * Get the filename extension
   *
   * @param filename The name of a file
   *
   * @return The extension
   */
gchar* 
gsampler_utils_get_extension (const gchar* filename) {
	gint offset = 0;
	gchar* ext = NULL;
	g_return_val_if_fail (filename != NULL, NULL);

	offset = string_last_index_of_char (filename, (gunichar) '.', 0);
	if (offset != (-1)) {
		gchar* temp = NULL;
		temp = string_substring (filename, (glong) (offset + 1), (glong) (-1));
		ext = g_utf8_strdown (temp, (gssize) (-1));
		g_free (temp);
	}
	return ext;
}


/**
   * Get the filename and Remove the extension
   *
   * @param filename The name of a file
   *
   * @return The name of the file without extension
   */
gchar* 
gsampler_utils_get_name (const gchar* filename) { 
	GFile* file = NULL;
	gchar* name = NULL;
	gchar* result = NULL;
	g_return_val_if_fail (filename != NULL, NULL);

	file = g_file_new_for_path (filename);
	name = g_file_get_basename (file);
	if (name != NULL) {
		gint offset = 0;
		offset = string_last_index_of_char (name, (gunichar) '.', 0);
		if (offset != (-1)) {
			result = g_strndup (name, (gsize) offset);
		}
	} else {
		result = g_strdup (filename);
	}
	g_free (name);
	g_object_unref (file);
	return result;
}

/**
   * Test if a file exist
   *
   * @return True if file exist....
   */
gboolean
gsampler_utils_file_exist (const gchar* file) {
	FILE *stream;
	
	if ((stream = fopen(file, "r")) == NULL)
	{	
		return FALSE;
	}
	fclose (stream);
	return TRUE;
}


gchar *
gsampler_utils_get_data_file (const gchar* file)
{
	gchar *test = NULL;
	gchar *ret = NULL;
	g_return_val_if_fail (file != NULL, NULL);

	test = g_strconcat (PACKAGE_DATA_DIR, "/", file, NULL);
	if (g_file_test (test, G_FILE_TEST_EXISTS))
	{
		ret = g_strdup (test);
	} else 
	{
		g_free (test);
		test = g_strconcat ("data/", file, NULL);
		if (g_file_test (test, G_FILE_TEST_EXISTS))
			ret = g_strdup (test);
	}
	g_free (test);
	return ret;
}

#ifdef G_OS_WIN32
gchar*		
gsampler_utils_convert_win_path (const gchar* file_path)
{
	gchar *result = NULL;
	g_return_val_if_fail (file_path != NULL, NULL);

	result = string_replace (file_path, "\\", "/");
	return result;
}
#endif

/******************************************************************************/
/* Les fonctions qui permettent la gestion des list en fct d'un id	          */
/******************************************************************************/

GSList *
g_id_list_add (GSList *list, gint id, gpointer data) {

	struct Struct_list_widget *Slist_widget = NULL;

	// Allocation mémoire d'une nouvelle structure. Le transtypage n'est
	// nécessaire que si vous compilez en C++.
	Slist_widget = (struct Struct_list_widget*) g_malloc (sizeof (struct Struct_list_widget));
	// Affection des valeurs
	Slist_widget->Sdata = data;
	Slist_widget->Sid = id;

	// Ajout des nouvelles données à la liste.
	list = g_slist_append (list, Slist_widget);

	return list;
}

GSList *
g_id_list_remove (GSList *list, gint id) {
	GSList *tmp_list = NULL;

	// Recherche de l'objet dans la liste
	tmp_list = list;
	while (tmp_list) {
		if (((struct Struct_list_widget*)(tmp_list->data))->Sid == id)	{
			// Supression de l'objet
			g_free ((struct Struct_list_widget*)(tmp_list->data));
			list = g_slist_remove_link(list, tmp_list);
			g_slist_free_1 (tmp_list);
			return list;
		}
		tmp_list = g_slist_next(tmp_list);
	}
	return NULL;
}

gpointer *
g_id_list_get (GSList *list, gint id) {
	GSList *tmp_list = NULL;

	// Recherche de l'objet dans la liste
	tmp_list = list;
	while (tmp_list) {
		if (((struct Struct_list_widget*) (tmp_list->data))->Sid == id)
			return ((struct Struct_list_widget*) (tmp_list->data))->Sdata;
		tmp_list = g_slist_next (tmp_list);
	}
	return NULL;
}

gint
g_id_list_get_first_id (GSList *list) {
	GSList *tmp_list = NULL;

	tmp_list = list;
	if (tmp_list->data != NULL)
		return ((struct Struct_list_widget*) (tmp_list->data))->Sid;
	return -1;
}

gboolean
g_id_list_exist(GSList *list, gint id) {
	GSList *tmp_list = NULL;

	// Recherche de l'objet dans la liste
	tmp_list = list;
	while (tmp_list) {
		gint test_id = ((struct Struct_list_widget*) (tmp_list->data))->Sid;
		if ( test_id == id)
			return TRUE;
		tmp_list = g_slist_next(tmp_list);
	}
	return FALSE;
}

gint
g_id_list_size (GSList *list) {
	GSList *tmp_list = NULL;
	gint size = 0;

	tmp_list = list;
	while (tmp_list) {
		size++;
		tmp_list = g_slist_next(tmp_list);
	}
	return size;
}

GSList*
g_id_list_remove_all(GSList *list) {
	GSList *tmp_list = NULL;

	tmp_list = list;
	while (tmp_list) {
		g_free ((struct Struct_list_widget*) (tmp_list->data));
		tmp_list = g_slist_next (tmp_list);
	}
	return list;
}

void
g_id_list_free (GSList *list, gboolean clear) {
	if (clear)
		list = g_id_list_remove_all(list);
	g_slist_free (list);
}

/*
 * For gtk 2.22 and less don't use of GtkComboBoxText
 */
GtkComboBox *
gsampler_text_combo_create (void)
{
	GtkComboBox *result = NULL;
	GtkListStore *list = NULL;
	GtkCellRenderer * render;

	list = gtk_list_store_new (1, G_TYPE_STRING); 
	result = (GtkComboBox*) gtk_combo_box_new_with_model ((GtkTreeModel*) list);
	render = gtk_cell_renderer_text_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (result), render, FALSE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (result), render, "text", 0, NULL);
	return result;
}

void
gsampler_text_combo_append (GtkComboBox *combo, const gchar* text)
{
	GtkTreeIter iter = {0};
	GtkListStore *model = NULL;
	g_return_if_fail (combo != NULL);
	g_return_if_fail (text != NULL);
	
	model = (GtkListStore*) gtk_combo_box_get_model (combo);
	gtk_list_store_append (model, &iter);
	gtk_list_store_set (model, &iter, 0, text, -1);
}

void
gsampler_text_combo_insert (GtkComboBox *combo, const gchar* text, gint position)
{
	GtkTreeIter iter = {0};
	GtkListStore *model = NULL;
	g_return_if_fail (combo != NULL);
	g_return_if_fail (text != NULL);
	
	model = (GtkListStore*) gtk_combo_box_get_model (combo);
	gtk_list_store_insert (model, &iter, position);
	gtk_list_store_set (model, &iter, 0, text, -1);
}

gchar *
gsampler_text_combo_get_active (GtkComboBox *combo)
{
	GtkTreeIter iter = {0};
	GtkTreeModel *model = NULL;
	gchar* result = NULL;
	g_return_val_if_fail (combo != NULL, NULL);

	model = gtk_combo_box_get_model (combo);
	if (gtk_combo_box_get_active_iter (combo, &iter))
	{
		gtk_tree_model_get (model, &iter, 0 , &result, -1);
	}
	return result;
}

void
gsampler_text_combo_set_active (GtkComboBox *combo, const gchar* text)
{
	GtkTreeIter iter = {0};
	GtkTreeModel *model = NULL;
	g_return_if_fail (combo != NULL);
	g_return_if_fail (text != NULL);

	model = gtk_combo_box_get_model (combo);
	if (gtk_tree_model_get_iter_first (model, &iter))
	{
		do
		{
			gchar* val = NULL;
			gtk_tree_model_get (model, &iter, 0, &val, -1);
			if (g_strcmp0 (val, text) == 0)
			{
				gtk_combo_box_set_active_iter (combo, &iter);
				g_free (val);
				return;
			}
			g_free (val);
		} while (gtk_tree_model_iter_next (model, &iter));
	}
}


void
gsampler_text_combo_remove_all (GtkComboBox *combo) 
{
	GtkListStore *model = NULL;
	g_return_if_fail (combo != NULL);
	
	model = (GtkListStore*) gtk_combo_box_get_model (combo);
	gtk_list_store_clear (model);
}

void
gsampler_text_combo_configure (GtkComboBox *combo)
{
	GtkListStore *list = NULL;
	GtkCellRenderer * render;
	g_return_if_fail (combo != NULL);

	list = gtk_list_store_new (1, G_TYPE_STRING);
	gtk_combo_box_set_model (combo, (GtkTreeModel*) list);
	render = gtk_cell_renderer_text_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), render, FALSE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), render, "text", 0, NULL);
}

#if GLIB_MINOR_VERSION < 28
void
g_slist_free_full (GSList *list, GDestroyNotify  free_func)
{
  g_slist_foreach (list, (GFunc) free_func, NULL);
  g_slist_free (list);
}
#endif